@@ -2,6 +2,8 @@ package Net::OAI::Record::Header;
 
 use strict;
 use base qw( XML::SAX::Base );
+use Carp qw( carp );
+our $VERSION = 'v1.00.0';
 
 =head1 NAME
 
@@ -20,12 +22,9 @@ Net::OAI::Record::Header - class for record header representation
 sub new {
     my ( $class, %opts ) = @_;
     my $self = bless \%opts, ref( $class ) || $class;
-    $self->{ status } = '';
-    $self->{ identifier	} = '';
-    $self->{ datestamp } = '';
+    $self->{ status } = $self->{ identifier } = $self->{ datestamp } = '';
     $self->{ sets } = [];
-    $self->{ insideHeader } = 0;
-    $self->{ insideSet } = 0;
+    $self->{ insideHeader } = $self->{ insideSet } = $self->{ insideMetadata } = 0;
     return( $self );
 }
 
@@ -35,8 +34,8 @@ sub new {
 
 sub status {
     my ( $self, $status ) = @_;
-    if ( $status ) { $self->{ status } = $status; }
-    return( $self->{ status } );
+    if ( $status ) { $self->{ headerStatus } = $status; }
+    return( $self->{ headerStatus } );
 }
 
 =head2 identifier()
@@ -69,50 +68,66 @@ sub sets {
     return( @{ $self->{ sets } } );
 }
 
+my $xmlns_oai = "http://www.openarchives.org/OAI/2.0/";
+
 ## SAX Handlers
 
 sub start_element {
     my ( $self, $element ) = @_;
-    if ( $element->{ Name } eq 'header' ) { 
+    return $self->SUPER::start_element($element) unless $element->{NamespaceURI} eq $xmlns_oai;
+
+    if ( $element->{ LocalName } eq 'record' ) { 
+	$self->{ insideHeader } = $self->{ insideSet } = $self->{ insideMetadata } = 0}
+    elsif ( $element->{ LocalName } eq 'header' ) { 
 	$self->{ insideHeader } = 1;
 	if ( exists( $element->{ Attributes }{ '{}status' } ) ) {
-	    $self->{ headerStatus } = $element->{ Attributes }{ '{}status' };
+	    $self->{ headerStatus } = 
+                $element->{ Attributes }{ '{}status' }{ Value };
 	} else {
 	    $self->{ headerStatus } = '';
 	}
     }
-    elsif ( $element->{ Name } eq 'setSpec' ) {
+    elsif ( $element->{ LocalName } eq 'setSpec' ) {
 	$self->{ insideSet } = 1;
     }
-    else {
-	$self->SUPER::start_element( $element );
+    elsif ( $element->{ LocalName } eq 'metadata' ) {
+	$self->{ insideMetadata } = 1;
     }
-    push( @{ $self->{ tagStack } }, $element->{ Name } );
+#    elsif ( $self->{ insideMetadata } ) {
+#	$self->SUPER::start_element( $element );
+#    }
+    push( @{ $self->{ tagStack } }, $element->{ LocalName } );
 }
 
 sub end_element {
     my ( $self, $element ) = @_;
-    my $tagName = $element->{ Name };
+    return $self->SUPER::end_element($element) unless $element->{NamespaceURI} eq $xmlns_oai;
+
+    my $tagName = $element->{ LocalName };
+
     if ( $tagName eq 'header' ) {
 	$self->{ insideHeader } = 0;
+        ($self->{header} =~ /\S/) && carp "Excess content in record header: ".$self->{ header };
     }
     elsif ( $tagName eq 'setSpec' ) { 
 	push( @{ $self->{ sets } }, $self->{ setSpec } );
 	$self->{ insideSet } = 0;
     }
-    else { 
-	$self->SUPER::end_element( $element );
+    elsif ( $element->{ LocalName } eq 'metadata' ) {
+	$self->{ insideMetadata } = 0;
     }
+#    elsif ( $self->{ insideMetadata } ) {
+#	$self->SUPER::end_element( $element );
+#   }
     pop( @{ $self->{ tagStack } } );
 }
 
 sub characters {
     my ( $self, $characters ) = @_;
     if ( $self->{ insideHeader } ) { 
-	$self->{ $self->{ tagStack }[-1] } .= $characters->{ Data };
-    } else {
-	$self->SUPER::characters( $characters );
-    }
+	$self->{ $self->{ tagStack }[-1] } .= $characters->{ Data }}
+    elsif ( $self->{ insideMetadata } ) { 
+	$self->SUPER::characters( $characters )}
 }
 
 1;
